@@ -8,7 +8,7 @@ mfunpl0 <- function(par,siq,grad,pen=1e2){
 lpar <- length(par)
 m <- (lpar-1)/2
 ngrad <- dim(grad)[1]
-.Fortran("mfunpl0",as.double(par),#par(lpar)
+erg <- .Fortran("mfunpl0",as.double(par),#par(lpar)
                 as.double(siq),#siq(ngrad)
                 as.double(t(grad)),#grad(3,ngrad)
                 as.integer(m),#number of components
@@ -19,6 +19,29 @@ ngrad <- dim(grad)[1]
                 double(ngrad),#w(ngrad) working array
                 erg = double(1),#residual sum of squares
                 PACKAGE="dti")$erg
+erg
+}
+mfunpl0h <- function(par,siq,grad){
+#
+#   evaluate rss for Mixtensor-model (without isotropic component)
+#   uses LawsonHanson-nnls code
+#
+lpar <- length(par)
+m <- (lpar-1)/2
+ngrad <- dim(grad)[1]
+erg <- .Fortran("mfunpl0h",as.double(par),#par(lpar)
+                as.double(siq),#siq(ngrad)
+                as.double(t(grad)),#grad(3,ngrad)
+                as.integer(m),#number of components
+                as.integer(lpar),#number of parameters
+                as.integer(ngrad),#number of gradient
+                double(ngrad*m),#z(ngrad,m) working array
+                double(ngrad),#w(ngrad) working array
+                double(ngrad),# b(ngrad) working array
+                double(ngrad),# work1(ngrad) working array                
+                erg = double(1),#residual sum of squares
+                PACKAGE="dti")$erg
+erg
 }
 gmfunpl0 <- function(par,siq,grad,pen=1e2){
 #
@@ -27,7 +50,7 @@ gmfunpl0 <- function(par,siq,grad,pen=1e2){
 lpar <- length(par)
 m <- (lpar-1)/2
 ngrad <- dim(grad)[1]
-.Fortran("mfunpl0g",
+dfdpar<-.Fortran("mfunpl0g",
          as.double(par),#par(lpar)
          as.double(siq),#s(n)
          as.double(t(grad)),#g(3,n)
@@ -37,7 +60,7 @@ ngrad <- dim(grad)[1]
          double(3*m),# d(3,m)
          double(ngrad*m),# z(n,m)
          double(m*m),# v(m,m)
-         double(m),# w(m)
+         double(ngrad),# w(n) need w((m+1):n) for solver dgelsy
          double(ngrad*m),# dkgj(n,m)
          double(ngrad*m),# dkgj2(n,m)
          double(ngrad*m),# ddkdphig(n,m)
@@ -47,8 +70,9 @@ ngrad <- dim(grad)[1]
          double(m*m),# dvdth(m,m)
          double(m*m*m),# dvdphi(m,m,m)
          double(m*m*m),# dvdeta(m,m,m)
-         double(ngrad*m*lpar),# dzdpars(n,m,lpar)
+         double(ngrad*m*3),# dzdpars(n,m,3)
          double(m*lpar),# dwdpars(m,lpar)
+         double(m*lpar),# dwdpars2(m,lpar)
          double(ngrad*m),# zs(n,m)
          double(ngrad*m),# work1(n,m)
          double(ngrad*m),# work2(n,m)
@@ -56,6 +80,57 @@ ngrad <- dim(grad)[1]
          as.double(pen),# pen
          dfdpar=double(lpar),# dfdpar(lpar)
          PACKAGE="dti")$dfdpar
+dfdpar
+}
+gmfunpl0n <- function(par,siq,grad,pen=1e2){
+#
+#   evaluate numeric gradient approximations for Mixtensor-model
+#
+lpar <- length(par)
+m <- (lpar-1)/2
+ngrad <- dim(grad)[1]
+eps <- 1.e-8
+.Fortran("mfpl0gn",
+                as.double(par),#par(lpar)
+                as.double(siq),#siq(ngrad)
+                as.double(t(grad)),#grad(3,ngrad)
+                as.integer(m),#number of components
+                as.integer(lpar),#number of parameters
+                as.integer(ngrad),#number of gradients
+                as.double(pen),#penalty for negative weights
+                as.double(eps),
+                double(ngrad*m),#z(ngrad,m) working array
+                double(ngrad),#w(ngrad) working array
+                double(lpar),
+                double(lpar),
+                dfdpar=double(lpar),#residual sum of squares
+                PACKAGE="dti")$dfdpar
+}
+gmfunpl0hn <- function(par,siq,grad){
+#
+#   evaluate numeric gradient approximations for Mixtensor-model
+#   uses LawsonHanson-nnls code
+#
+lpar <- length(par)
+m <- (lpar-1)/2
+ngrad <- dim(grad)[1]
+eps <- 1.e-5
+.Fortran("mfpl0hgn",
+                as.double(par),#par(lpar)
+                as.double(siq),#siq(ngrad)
+                as.double(t(grad)),#grad(3,ngrad)
+                as.integer(m),#number of components
+                as.integer(lpar),#number of parameters
+                as.integer(ngrad),#number of gradients
+                as.double(eps),
+                double(ngrad*m),#z(ngrad,m) working array
+                double(ngrad),#w(ngrad) working array
+                double(ngrad),#b(ngrad) working array
+                double(ngrad),#work1(ngrad) working array
+                double(lpar),#  par - eps e_i
+                double(lpar),#  par + eps e_i
+                dfdpar=double(lpar),#residual sum of squares
+                PACKAGE="dti")$dfdpar
 }
 mfunplwghts0 <- function(par,siq,grad,pen=1e2){
 #
@@ -76,12 +151,10 @@ w<-.Fortran("mfunpl0",as.double(par),#par(lpar)
                 w = double(ngrad),#w(ngrad) working array
                 double(1),#residual sum of squares
                 PACKAGE="dti")$w[1:m]
-#                if(any(is.na(w)|abs(w)>1e10)){
-#                   cat("par",par,"\n siq",siq,"\n m",m,"\n lpar",lpar,"\n pen",
-#                        pen,"\n w",w,"\n")
-#                }
+                w <- pmax(0,w)
+                if(all(is.finite(w))) {
 erg<-.Fortran("mfunpl0w",as.double(par),#par(lpar)
-                as.double(pmax(w,0)),
+                as.double(w),
                 as.double(siq),#siq(ngrad)
                 as.double(t(grad)),#grad(3,ngrad)
                 as.integer(m),#number of components
@@ -90,6 +163,54 @@ erg<-.Fortran("mfunpl0w",as.double(par),#par(lpar)
                 double(ngrad*m),#z(ngrad,m) working array
                 erg = double(1),#residual sum of squares
                 PACKAGE="dti")$erg
+           } else {
+           cat("got w=",w,"\n")
+           erg <- 1e20
+           }
+           o <- order(w,decreasing=TRUE)
+           ord <- sum(w>0)
+           if(ord<m){
+              o <- o[1:ord]
+           }
+           sw <- sum(w[w>0])
+           lev <- c(par[1],-log(sw))
+           if(ord>0){
+           mix <- w[o]/sw
+           } else {
+           mix <- NULL
+           } 
+           or <- matrix(par[2:lpar],2,m)[,o,drop=FALSE]
+           or[1,or[1,]<0] <- or[1,or[1,]<0]+pi
+           or[1,or[1,]>pi] <- or[1,or[1,]>pi]-pi
+           or[2,or[2,]<0] <- or[2,or[2,]<0]+2*pi
+           or[2,or[2,]>2*pi] <- or[2,or[2,]>2*pi]-2*pi
+           par <- c(par[1],or[,1:ord])
+list(ord=ord,lev=lev,mix=mix,orient=or,par=par,value=erg)
+}
+mfunplwghts0h <- function(par,siq,grad,pen=1e2){
+#
+#   get weights for Mixtensor-model (without isotropic component) and extract parameters 
+#
+par[1] <- max(0,par[1])
+#   uses LawsonHanson-nnls code
+#
+lpar <- length(par)
+m <- (lpar-1)/2
+ngrad <- dim(grad)[1]
+z <- .Fortran("mfunpl0h",as.double(par),#par(lpar)
+                as.double(siq),#siq(ngrad)
+                as.double(t(grad)),#grad(3,ngrad)
+                as.integer(m),#number of components
+                as.integer(lpar),#number of parameters
+                as.integer(ngrad),#number of gradients
+                double(ngrad*m),#z(ngrad,m) working array
+                w=double(ngrad),#w(ngrad) working array
+                double(ngrad),# b(ngrad) working array
+                double(ngrad),# work1(ngrad) working array                
+                erg = double(1),#residual sum of squares
+                PACKAGE="dti")[c("erg","w")]
+           erg <- z$erg
+           w <- z$w
            o <- order(w,decreasing=TRUE)
            ord <- sum(w>0)
            if(ord<m){
@@ -102,7 +223,6 @@ erg<-.Fortran("mfunpl0w",as.double(par),#par(lpar)
            } else {
            mix <- NULL
            } 
-#           if(par[1]<0) cat("par",par,"\n")
            or <- matrix(par[2:lpar],2,m)[,o,drop=FALSE]
            or[1,or[1,]<0] <- or[1,or[1,]<0]+pi
            or[1,or[1,]>pi] <- or[1,or[1,]>pi]-pi
@@ -122,10 +242,10 @@ lpar <- length(par)
 m <- (lpar-1)/2
 mp1 <- m+1
 ngrad <- dim(grad)[1]
+#cat("par:",par,"\n")
 z<-.Fortran("mfunpl1",as.double(par),#par(lpar)
                 as.double(siq),#siq(ngrad)
                 as.double(t(grad)),#grad(3,ngrad)
-                as.integer(m),#number of components
                 as.integer(mp1),#number of components+1
                 as.integer(lpar),#number of parameters
                 as.integer(ngrad),#number of gradients
@@ -134,7 +254,7 @@ z<-.Fortran("mfunpl1",as.double(par),#par(lpar)
                 double(ngrad),#w(ngrad) working array
                 erg = double(1),#residual sum of squares
                 PACKAGE="dti")$erg
-#         cat("par:",signif(par,3),"fw:",signif(z,3),"\n")
+#cat("erg:",z,"\n")
 z
 }
 gmfunpl1 <- function(par,siq,grad,pen=1e2){
@@ -145,6 +265,8 @@ lpar <- length(par)
 m <- (lpar-1)/2
 mp1 <- m+1
 ngrad <- dim(grad)[1]
+#cat("gmfunpl1:par",par,"m",m,"mp1",mp1,"lpar",lpar,"ngrad",
+#ngrad,"dim(grad)",dim(grad),"\n siq",siq,"\n")
 z<-.Fortran("mfunpl1g",
          as.double(par),#par(lpar)
          as.double(siq),#s(n)
@@ -156,7 +278,7 @@ z<-.Fortran("mfunpl1g",
          double(3*m),# d(3,m)
          double(ngrad*mp1),# z(n,mp1)
          double(mp1*mp1),# v(mp1,mp1)
-         double(mp1),# w(mp1)
+         double(ngrad),# w(ngrad)
          double(ngrad*m),# dkgj(n,m)
          double(ngrad*m),# dkgj2(n,m)
          double(ngrad*m),# ddkdphig(n,m)
@@ -166,8 +288,9 @@ z<-.Fortran("mfunpl1g",
          double(mp1*mp1),# dvdth(mp1,mp1)
          double(mp1*mp1*m),# dvdphi(mp1,mp1,m)
          double(mp1*mp1*m),# dvdeta(mp1,mp1,m)
-         double(ngrad*mp1*lpar),# dzdpars(n,mp1,lpar)
+         double(ngrad*mp1*3),# dzdpars(n,mp1,3)
          double(mp1*lpar),# dwdpars(mp1,lpar)
+         double(mp1*lpar),# dwdpars2(mp1,lpar)
          double(ngrad*mp1),# zs(n,mp1)
          double(ngrad*mp1),# work1(n,mp1)
          double(ngrad*mp1),# work2(n,mp1)
@@ -175,10 +298,34 @@ z<-.Fortran("mfunpl1g",
          as.double(pen),# pen
          dfdpar=double(lpar),# dfdpar(lpar)
          PACKAGE="dti")$dfdpar
-#         cat("par:",signif(par,3),"\n")
-#         cat("gradient:",signif(z,3),"\n")
+#cat("returned gradient",z,"\n")
          z
 }
+gmfunpl1n <- function(par,siq,grad,pen=1e2){
+#
+#   evaluate numeric gradient approximations for Mixtensor-model
+#
+lpar <- length(par)
+m <- (lpar+1)/2
+# this actually refers to number of components + 1
+ngrad <- dim(grad)[1]
+eps <- 1.e-8
+.Fortran("mfpl1gn",
+                as.double(par),#par(lpar)
+                as.double(siq),#siq(ngrad)
+                as.double(t(grad)),#grad(3,ngrad)
+                as.integer(m),#number of components
+                as.integer(lpar),#number of parameters
+                as.integer(ngrad),#number of gradients
+                as.double(pen),#penalty for negative weights
+                as.double(eps),
+                double(ngrad*m),#z(ngrad,m) working array
+                double(ngrad),#w(ngrad) working array
+                double(lpar),
+                double(lpar),
+                dfdpar=double(lpar),#residual sum of squares
+                PACKAGE="dti")$dfdpar
+}
 mfunplwghts1 <- function(par,siq,grad,pen=1e2){
 #
 #   get weights for Mixtensor-model (without isotropic component) and extract parameters 
@@ -187,10 +334,10 @@ lpar <- length(par)
 m <- (lpar-1)/2
 mp1 <- m+1
 ngrad <- dim(grad)[1]
+#cat("mfunplwghts1:par",par,"\n")
 w<-.Fortran("mfunpl1",as.double(par),#par(lpar)
                 as.double(siq),#siq(ngrad)
                 as.double(t(grad)),#grad(3,ngrad)
-                as.integer(m),#number of components
                 as.integer(mp1),#number of components+1
                 as.integer(lpar),#number of parameters
                 as.integer(ngrad),#number of gradients
@@ -199,17 +346,18 @@ w<-.Fortran("mfunpl1",as.double(par),#par(lpar)
                 w = double(ngrad),#w(ngrad) working array
                 double(1),#residual sum of squares
                 PACKAGE="dti")$w[1:mp1]
+#cat("mfunplwghts1:w",w,"\n")
            erg <- .Fortran("mfunpl1w",as.double(par),#par(lpar)
                 as.double(pmax(w,0)),#par(lpar) 
                 as.double(siq),#siq(ngrad)
                 as.double(t(grad)),#grad(3,ngrad)
-                as.integer(m),#number of components
                 as.integer(mp1),#number of components+1
                 as.integer(lpar),#number of parameters
                 as.integer(ngrad),#number of gradients
                 double(ngrad*mp1),#z(ngrad,m+1) working array
                 erg = double(1),#residual sum of squares
                 PACKAGE="dti")$erg
+#cat("mfunplwghts1:erg",erg,"\n")
            w0 <- w[1]
            w <- w[-1]
            o <- order(w,decreasing=TRUE)
@@ -219,8 +367,6 @@ w<-.Fortran("mfunpl1",as.double(par),#par(lpar)
            }
            problem <- FALSE
            if(sum(w[w>0])+w0<=0){
-#           cat("w0",w0,"w",w,"\n")
-#           cat("par",par,"\n")
            problem <- TRUE
            }
            sw <- sum(w[w>0])+max(w0,0)
@@ -259,9 +405,6 @@ ind <- .Fortran("selisamp",
                 as.double(maxc),
                 DUPL=FALSE,
                 PACKAGE="dti")$ind 
-#for(i in 1:nguess) for(j in 1:(maxcomp-1)) for(k in (j+1):maxcomp){
-#if(dgrad[isample[j,i],isample[k,i]]>maxc) ind[i] <- FALSE
-#}
 .Random.seed <- saved.seed
 }
 isample[,ind]
@@ -395,39 +538,66 @@ krit[landir] <- z$krit[landir]
 }
 failed <- (krit^2/ngrad) > (sigma2-1e-10)
 if(any(failed[mask])){
-print((krit[mask])[failed[mask]])
-print(((1:prod(dim(si)[1:3]))[mask])[failed[mask]])
-print(sum(failed[mask]))
+#print((krit[mask])[failed[mask]])
+#print(((1:prod(dim(si)[1:3]))[mask])[failed[mask]])
+#print(sum(failed[mask]))
 }
 list(siind=array(siind,c(maxcomp+2,dim(si)[-4])),
      krit=array(krit,dim(si)[-4]))
 }
-
-getsiind2 <- function(si,mask,sigma2,grad,vico,th,maxcomp=3,maxc=.866,nguess=100){
+getsiind3iso <- function(si,mask,sigma2,grad,vico,th,indth,ev,fa,andir,maxcomp=3,maxc=.866,nguess=100){
 # assumes dim(grad) == c(ngrad,3)
 # assumes dim(si) == c(n1,n2,n3,ngrad)
 # SO removed
 ngrad <- dim(grad)[1]
 nvico <- dim(vico)[1]
+ddim <- dim(fa)
 nsi <- dim(si)[4]
 dgrad <- matrix(abs(grad%*%t(vico)),ngrad,nvico)
 dgrad <- dgrad/max(dgrad)
 dgradi <- matrix(abs(vico%*%t(vico)),nvico,nvico)
 dgradi <- dgradi/max(dgradi)
 nth <- length(th)
-isample <- selisample(nvico,maxcomp,nth*nguess,dgradi,maxc)
+nvoxel <- prod(ddim)
+landir <- fa>.3
+landir[is.na(landir)] <- FALSE
+if(any(is.na(andir))) {
+cat(sum(is.na(andir)),"na's in andir")
+andir[is.na(andir)]<-sqrt(1/3)
+}
+if(any(is.na(landir))) {
+cat(sum(is.na(landir)),"na's in landir")
+landir[is.na(landir)]<-0
+}
+if(any(is.na(fa))) {
+cat(sum(is.na(fa)),"na's in fa")
+fa[is.na(fa)]<-0
+}
+iandir <- array(.Fortran("iandir",
+                   as.double(t(vico)),
+                   as.integer(nvico),
+                   as.double(andir),
+                   as.integer(nvoxel),
+                   as.logical(landir),
+                   iandir=integer(2*prod(ddim)),
+                   DUPL=FALSE,
+                   PACKAGE="dti")$iandir,c(2,nvoxel))
+isample0 <- selisample(nvico,maxcomp,nguess,dgradi,maxc)
+if(maxcomp>1) isample1 <- selisample(nvico,maxcomp-1,nguess,dgradi,maxc)
+if(maxcomp==1) isample1 <- sample(ngrad, nguess, replace = TRUE)
 #
 #  eliminate configurations with close directions 
 #
-if(maxcomp>1) { 
-nguess <- trunc(dim(isample)[2]/nth) 
-isample <- array(isample[,1:(nguess*nth)],c(maxcomp,nguess,nth))
-} 
 # this provides configurations of initial estimates with minimum angle between 
 # directions > acos(maxc)
-cat("using ",nguess,"guesses for initial estimates\n")
 nvoxel <- prod(dim(si)[-4])
-siind <- .Fortran("getsiin2",
+cat("using ",nguess,"guesses for initial estimates\n")
+siind <- matrix(as.integer(0),maxcomp+2,nvoxel)
+krit <- numeric(nvoxel)
+# first voxel with fa<.3
+cat(sum(mask&!landir),"voxel with small FA\n")
+nguess <- length(isample0)/maxcomp
+z <- .Fortran("getsi30i",
          as.double(aperm(si,c(4,1:3))),
          as.double(sigma2),
          as.integer(nsi),
@@ -437,26 +607,63 @@ siind <- .Fortran("getsiin2",
          as.integer(nvico),
          as.double(th),
          as.integer(nth),
+         as.integer(indth),
          double(ngrad*nvico),
-         as.integer(isample),
+         as.integer(isample0),
          as.integer(nguess),
          double(nsi),
-         double(nsi*(maxcomp+2)),
+         double(nsi*(maxcomp+3)),
          siind=integer((maxcomp+2)*nvoxel),
-         krit=double(prod(dim(si)[1:3])),
+         krit=double(nvoxel),
          as.integer(maxcomp+2),
-         as.logical(mask),
+         as.logical(mask&!landir),
          PACKAGE="dti")[c("siind","krit")]
-failed <- (siind$krit^2/ngrad) > (sigma2-1e-10)
+dim(z$siind) <- c(maxcomp+2,nvoxel)
+siind[,!landir] <- z$siind[,!landir]
+krit[!landir] <- z$krit[!landir]
+# now voxel where first tensor direction seems important
+if(maxcomp >0){
+cat(sum(mask&landir),"voxel with distinct first eigenvalue \n")
+nguess <- if(maxcomp>1) length(isample1)/(maxcomp-1) else length(isample1)
+z <- .Fortran("getsi31i",
+         as.double(aperm(si,c(4,1:3))),
+         as.double(sigma2),
+         as.integer(nsi),
+         as.integer(nvoxel),
+         as.integer(maxcomp),
+         as.double(dgrad),
+         as.integer(nvico),
+         as.integer(iandir[1,]),
+         as.double(th),
+         as.integer(nth),
+         as.integer(indth),
+         double(ngrad*nvico),
+         as.integer(isample1),
+         as.integer(nguess),
+         double(nsi),
+         double(nsi*(maxcomp+3)),
+         siind=integer((maxcomp+2)*nvoxel),
+         krit=double(nvoxel),
+         as.integer(maxcomp+2),
+         as.logical(mask&landir),
+         as.double(dgradi),
+         as.double(maxc),
+         PACKAGE="dti")[c("siind","krit")]
+dim(z$siind) <- c(maxcomp+2,nvoxel)
+siind[,landir] <- z$siind[,landir]
+krit[landir] <- z$krit[landir]
+}
+failed <- (krit^2/ngrad) > (sigma2-1e-10)
 if(any(failed[mask])){
-print((siind$krit[mask])[failed[mask]])
-print(((1:prod(dim(si)[1:3]))[mask])[failed[mask]])
-print(sum(failed[mask]))
+#print((krit[mask])[failed[mask]])
+#print(((1:prod(dim(si)[1:3]))[mask])[failed[mask]])
+#print(sum(failed[mask]))
 }
-list(siind=array(siind$siind,c(maxcomp+2,dim(si)[-4])),
-     krit=array(siind$krit,dim(si)[-4]))
+list(siind=array(siind,c(maxcomp+2,dim(si)[-4])),
+     krit=array(krit,dim(si)[-4]))
 }
 
+
 dwiMixtensor <- function(object, ...) cat("No dwiMixtensor calculation defined for this class:",class(object),"\n")
 
 setGeneric("dwiMixtensor", function(object,  ...) standardGeneric("dwiMixtensor"))
@@ -465,10 +672,14 @@ setMethod("dwiMixtensor","dtiData",function(object, maxcomp=3,  p=40, method="mi
 #
 #  uses  S(g)/s_0 = w_0 exp(-l_1) +\sum_{i} w_i exp(-l_2-(l_1-l_2)(g^T d_i)^2)
 #
-#  
-#
+#  choices for optmethod:
+#  BFGS  -  BFGS with analytic gradients and penalization
+#  BFGSn -  BFGS with numeric gradients and penalization
+#  BFGSh -  L-BFGS-B using the LawsonHanson-nnls code to
+#           get nonnegative weights and lower bound 0 for theta
+#  else  -  Nelder-Mead on using LawsonHanson-nnls code
   set.seed(1)
-  if(is.null(pen)) pen <- 1e2
+  if(is.null(pen)) pen <- 100
   theta <- .5
   maxc <- .866
   args <- sys.call(-1)
@@ -512,8 +723,6 @@ setMethod("dwiMixtensor","dtiData",function(object, maxcomp=3,  p=40, method="mi
   th <- ev[1,,,] - (ev[2,,,]+ev[3,,,])/2
   falevel <- min(quantile(fa[fa>0],.75),.3)
   rth <- quantile(th[fa>=falevel],c(.1,.99))
-  th[th<rth[1]] <- rth[1]
-  th[th>rth[2]] <- rth[2]
   if(diff(rth)>0){
      indth <- trunc((nth-1)*(th-rth[1])/diff(rth)+1)
      th <- seq(rth[1],rth[2],length=nth)
@@ -521,6 +730,7 @@ setMethod("dwiMixtensor","dtiData",function(object, maxcomp=3,  p=40, method="mi
      th <- rep(max(th),nth)
      indth <- rep(1,length(th))
   }
+cat("using th:::",th,"\n")
   cat("Start search outlier detection at",format(Sys.time()),"\n")
 #
 #  replace physically meaningless S_i by mena S_0 values
@@ -595,7 +805,7 @@ setMethod("dwiMixtensor","dtiData",function(object, maxcomp=3,  p=40, method="mi
 #
 #  compute initial estimates (EV from grid and orientations from icosa3$vertices)
 #
-  siind <- getsiind3(siq,mask,sigma2,grad,t(vert),th,indth,ev,fa,andir,maxcomp,maxc=maxc,nguess=nguess)
+  siind <- if(method=="mixtensor")  getsiind3(siq,mask,sigma2,grad,t(vert),th,indth,ev,fa,andir,maxcomp,maxc=maxc,nguess=nguess) else getsiind3iso(siq,mask,sigma2,grad,t(vert),th,indth,ev,fa,andir,maxcomp,maxc=maxc,nguess=nguess)
   krit <- siind$krit # sqrt(sum of squared residuals) for initial estimates
   siind <- siind$siind # components 1: model order 2: 
                        # grid index for EV 2+(1:m) index of orientations
@@ -656,9 +866,9 @@ setMethod("dwiMixtensor","dtiData",function(object, maxcomp=3,  p=40, method="mi
            if(optmethod=="BFGS"){
                  z <- optim(par[1:(2*k+1)],mfunpl0,gmfunpl0,siq=siq[i1,i2,i3,],grad=grad,pen=pen,
                          method="BFGS",control=list(maxit=maxit,reltol=reltol))
-           } else {
-              z <- optim(par[1:(2*k+1)],mfunpl0,siq=siq[i1,i2,i3,],grad=grad,pen=pen,
-                         method=optmethod,control=list(maxit=maxit,reltol=reltol))
+           }  else {
+              z <- optim(par[1:(2*k+1)],mfunpl0,siq=siq[i1,i2,i3,],grad=grad,
+                         method="Nelder-Mead",control=list(maxit=maxit,reltol=reltol))
            }
         } else if (method=="mixtensoriso"){
            lpar <- 2*k+1
@@ -679,11 +889,18 @@ setMethod("dwiMixtensor","dtiData",function(object, maxcomp=3,  p=40, method="mi
         } else if (method=="mixtensoriso"){
             zz <- mfunplwghts1(z$par[1:lpar],siq[i1,i2,i3,],grad,pen)
         }
+#        gmfn <- gmfunpln(z$par[1:lpar],siq[i1,i2,i3,],grad,pen)    
+#        gmf0 <- gmfunpl0(z$par[1:lpar],siq[i1,i2,i3,],grad,pen)
+#        if(any(abs(gmfn-gmf0)>1e-2)){
+#           cat("gmfn",signif(gmfn,3),"\n",
+#               "gmf0",signif(gmf0,3),"\n",
+#               "neg w",k-zz$ord,"ev",zz$lev,"mix",zz$mix,"\n")
+#        }
         ord <- zz$ord
 #  replace sigmai by best variance estimate from currently best model
         value <- zz$value 
 # thats sum of squared residuals for the restricted model (w>0)
-        if(any(zz$lev<0)||ord<k){
+if(any(zz$lev<0)||ord<k){
            ttt <- krit
 #   parameters not interpretable reduce order
         } else {
