@@ -109,7 +109,8 @@ if(is.na(maxcomp)) maxcomp <- 3 else maxcomp <- min(5,max(1,maxcomp))
 z0 <- truemixtens(mix,th,alpha,beta,grad,sigma,ns0)
 z <- tdatamixtens(mix,th,alpha,beta,grad,sigma,ns0)
 zt <- dtiTensor(z)
-zmix <- dwiMixtensor(z,optmethod="BFGS",maxcomp=maxcomp,reltol=1e-6)
+zmix <- dwiMixtensor(z,maxcomp=maxcomp)
+zmix <- dwiMtImprove(zmix,z,maxcomp=maxcomp)
 gslexists <- "gsl" %in% .packages(TRUE)
 if(gslexists) zqball <- dwiQball(z,order=8,lambda=2e-2)
 
