@@ -11,8 +11,8 @@ class Converter
 		Voxel* voxels;
 		
 	public:
-		Converter(double*, double*, int, int, int);
-		Converter(double*, double*, int*,int, int, int, int);
+		Converter(double*, double*, int*, int, int, int);
+		Converter(double*, double*, int*, int*,int, int, int, int);
 		Voxel& getVoxels();
 };
 
